/*	$Id: netshring.c 2007/07/05 19:11:11 mallet $ */

/* Copyright (c) 2007 Anthony Mallet                  --  Mon Jun 18 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
static const char id[] = "$Id: netshring.c 2007/07/05 19:11:11 mallet $";

static const char test1[] = "Le petit lapin de Playboy";
static const char test2[] = "Ronge mon crne vgtal";
static const char test3[] = "Shoe shine boy";

#include <err.h>

#include "netshring.h"

#define PUSH(x, expected)						\
  s = netshring_push(prod, x, sizeof(x));				\
  if (s != (expected)) {						\
    warnx("push failed, returned %d instead of %d", s, (expected));	\
    netshring_disconnect(cons);						\
    netshring_delete(prod);						\
    return 2;								\
  }									\
  warnx("pushed `%s'", x)

#define POP(x, expected)					\
  s = netshring_pop(cons, x, sizeof(x), SHRNG_ASYNC);		\
  if (s != (expected)) {					\
    warnx("pop failed");					\
    netshring_disconnect(cons);					\
    netshring_delete(prod);					\
    return 2;							\
  }								\
  warnx("poped `%s'", x)

int
main()
{
  char r[1024];
  netshring_id_t prod, cons;
  int s;

  prod = netshring_create("test");
  if (!prod) {
    warnx("producer creation failed");
    return 2;
  }

  cons = netshring_connect("test", SHRNG_SYNC);
  if (!cons) {
    warnx("consumer creation failed");
    netshring_delete(prod);
    return 2;
  }

  /* simple sequence */
  PUSH(test1, sizeof(test1));
  POP(r, sizeof(test1));

  /* double sequence */
  PUSH(test1, sizeof(test1));
  PUSH(test2, sizeof(test2));
  POP(r, sizeof(test1));
  POP(r, sizeof(test2));

  /* three push */
  PUSH(test1, sizeof(test1));
  PUSH(test2, sizeof(test2));
  PUSH(test3, sizeof(test3));
  POP(r, sizeof(test1));
  POP(r, sizeof(test2));
  POP(r, sizeof(test3));

  /* empty pop must fail */
  r[0] = '\0';
  POP(r, 0);

  netshring_disconnect(cons);
  netshring_delete(prod);
  return 0;
}
